#include <iostream>
#include "channelflow/matrixutils.h"

using namespace std;
using namespace channelflow;

int main() {
  
  const int Nx = 12;
  const int Ny = 25;
  const int Nz = 32;
  const int Nd = 3;
  const Real Lx = 2*pi;
  const Real Lz = pi;
  const Real Reynolds = 400;
  const Real dtarg = 0.15;
  /***/ Real T = 20;
  const Real delta = 0.15;
  const bool verbose = true;

  FlowField u(Nx,Ny,Nz,Nd,Lx,Lz, -1,1, Spectral);
  u.cmplx(0,1,0,0) = Complex(-0.0869479, 0);
  u.cmplx(0,3,0,0) = Complex(0.110579, 0);
  u.cmplx(0,5,0,0) = Complex(-0.0281658, 0);
  u.cmplx(0,7,0,0) = Complex(0.00480068, 0);
  u.cmplx(0,0,1,0) = Complex(0, 0.0933982);
  u.cmplx(0,2,1,0) = Complex(0, -0.0986711);
  u.cmplx(0,1,2,0) = Complex(-0.00564985, 0);
  u.cmplx(0,3,2,0) = Complex(0.00746327, 0);
  u.cmplx(0,2,3,0) = Complex(0, 0.004087);
  u.cmplx(1,2,1,0) = Complex(0, -0.00472544);
  u.cmplx(11,2,1,0) = Complex(0, 0.00472544);
  u.cmplx(1,4,0,2) = Complex(0, 0.00401234);
  u.cmplx(11,4,0,2) = Complex(0, -0.00401234);


  Real CFL;
  TimeStep dt(dtarg, 0.01, 0.2, 1.0, 0.40, 0.80, false);
  SymmetryList symms(2);
  symms[0] = FieldSymmetry(1,  1, -1, 0.5, 0.5);
  symms[1] = FieldSymmetry(-1, -1,  1, 0.5, 0.5);



  DNSFlags dnsflags;
  dnsflags.baseflow     = LaminarBase;
  dnsflags.ulowerwall   = -1;
  dnsflags.uupperwall   =  1;
  dnsflags.nu           = 1.0/Reynolds;
  dnsflags.verbosity    = Silent;
  dnsflags.dt           = dt;
  dnsflags.symmetries   = symms;

  GMRESHookstepFlags srchflags;
  srchflags.delta = delta;
  srchflags.epsSearch = 1e-13;
  PoincareCondition* hpoincare = 0; // null (no) Poincare section condition
  
  FieldSymmetry sigma; //identity

  cerr << "findsolnTest: " << flush;
  if (verbose) {
    cout << "\n====================================================" << endl;
    cout << "chebyTest\n\n";
  }
  srchflags.logstream = &cout;
  
  Real residual = GMRESHookstep(u, T, sigma, hpoincare, srchflags, dnsflags, dt, CFL);

  Real normerr = abs(L2Norm(u) - 0.231007);
  Real disserr = abs(dissipation(u) - 0.413474);
  Real maxerr = 1e-06;

  Real error = residual + normerr + disserr;

  if (error < maxerr) {
    cerr << "\t   pass   " << endl;
    cout << "\t   pass   " << endl;
    return 0;
  }
  else {
    cerr << "\t** FAIL **" << endl;
    cout << "\t** FAIL **" << endl;
    cout << "error  == " << error << endl;
    cout << "maxerr == " << maxerr << endl;
    return 1;
  }
}
